@@ -53,8 +53,6 @@ CONF.register_opts(collector_monasca_opts, COLLECTOR_MONASCA_OPTS)
 ks_loading.register_auth_conf_options(CONF, COLLECTOR_MONASCA_OPTS)
 ks_loading.register_session_conf_options(CONF, COLLECTOR_MONASCA_OPTS)
 
-METRICS_CONF = ck_utils.load_conf(CONF.collect.metrics_conf)
-
 MONASCA_EXTRA_SCHEMA = {
     Required('extra_args', default={}): {
         # Key corresponding to the resource id in a metric's dimensions
@@ -179,7 +177,11 @@ class MonascaCollector(collector.BaseCollector):
         extra_args = self.conf[metric_name]['extra_args']
         kwargs = {}
         if extra_args['forced_project_id']:
-            kwargs['tenant_id'] = extra_args['forced_project_id']
+            if extra_args['forced_project_id'] == 'SCOPE_ID' and project_id:
+                kwargs['tenant_id'] = project_id
+                dimensions.pop(CONF.collect.scope_key, None)
+            else:
+                kwargs['tenant_id'] = extra_args['forced_project_id']
 
         return self._conn.metrics.list_statistics(
             name=metric_name,
