@@ -13,8 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Martin CAMEY
-#
 from decimal import Decimal
 
 import mock
@@ -23,9 +21,9 @@ from cloudkitty import collector
 from cloudkitty.collector import exceptions
 from cloudkitty.collector import prometheus
 from cloudkitty.common.prometheus_client import PrometheusResponseError
+from cloudkitty import dataframe
 from cloudkitty import tests
 from cloudkitty.tests import samples
-from cloudkitty import transformer
 
 
 class PrometheusCollectorTest(tests.TestCase):
@@ -48,16 +46,90 @@ class PrometheusCollectorTest(tests.TestCase):
                             'instance',
                         ],
                         'extra_args': {
-                            'aggregation_method': 'avg',
+                            'aggregation_method': 'avg'
                         },
                     },
                 }
             }
         }
-        transformers = transformer.get_transformers()
-        self.collector = prometheus.PrometheusCollector(transformers, **args)
+        args_range_function = {
+            'period': 3600,
+            'scope_key': 'namespace',
+            'conf': {
+                'metrics': {
+                    'http_requests_total': {
+                        'unit': 'instance',
+                        'groupby': [
+                            'foo',
+                            'bar',
+                        ],
+                        'metadata': [
+                            'code',
+                            'instance',
+                        ],
+                        'extra_args': {
+                            'aggregation_method': 'avg',
+                            'query_function': 'abs',
+                        },
+                    },
+                }
+            }
+        }
+        args_query_function = {
+            'period': 3600,
+            'scope_key': 'namespace',
+            'conf': {
+                'metrics': {
+                    'http_requests_total': {
+                        'unit': 'instance',
+                        'groupby': [
+                            'foo',
+                            'bar',
+                        ],
+                        'metadata': [
+                            'code',
+                            'instance',
+                        ],
+                        'extra_args': {
+                            'aggregation_method': 'avg',
+                            'range_function': 'delta',
+                        },
+                    },
+                }
+            }
+        }
+        args_all = {
+            'period': 3600,
+            'scope_key': 'namespace',
+            'conf': {
+                'metrics': {
+                    'http_requests_total': {
+                        'unit': 'instance',
+                        'groupby': [
+                            'foo',
+                            'bar',
+                        ],
+                        'metadata': [
+                            'code',
+                            'instance',
+                        ],
+                        'extra_args': {
+                            'aggregation_method': 'avg',
+                            'range_function': 'delta',
+                            'query_function': 'abs',
+                        },
+                    },
+                }
+            }
+        }
+        self.collector_mandatory = prometheus.PrometheusCollector(**args)
+        self.collector_without_range_function = prometheus.PrometheusCollector(
+            **args_range_function)
+        self.collector_without_query_function = prometheus.PrometheusCollector(
+            **args_query_function)
+        self.collector_all = prometheus.PrometheusCollector(**args_all)
 
-    def test_fetch_all_build_query(self):
+    def test_fetch_all_build_query_only_mandatory(self):
         query = (
             'avg(avg_over_time(http_requests_total'
             '{project_id="f266f30b11f246b589fd266f85eeec39"}[3600s]'
@@ -67,7 +139,7 @@ class PrometheusCollectorTest(tests.TestCase):
         with mock.patch.object(
             prometheus.PrometheusClient, 'get_instant',
         ) as mock_get:
-            self.collector.fetch_all(
+            self.collector_mandatory.fetch_all(
                 'http_requests_total',
                 samples.FIRST_PERIOD_BEGIN,
                 samples.FIRST_PERIOD_END,
@@ -75,7 +147,70 @@ class PrometheusCollectorTest(tests.TestCase):
             )
             mock_get.assert_called_once_with(
                 query,
+                samples.FIRST_PERIOD_END.isoformat(),
+            )
+
+    def test_fetch_all_build_query_without_range_function(self):
+        query = (
+            'avg(abs(avg_over_time(http_requests_total'
+            '{project_id="f266f30b11f246b589fd266f85eeec39"}[3600s]'
+            '))) by (foo, bar, project_id, code, instance)'
+        )
+
+        with mock.patch.object(
+            prometheus.PrometheusClient, 'get_instant',
+        ) as mock_get:
+            self.collector_without_range_function.fetch_all(
+                'http_requests_total',
+                samples.FIRST_PERIOD_BEGIN,
                 samples.FIRST_PERIOD_END,
+                self._tenant_id,
+            )
+            mock_get.assert_called_once_with(
+                query,
+                samples.FIRST_PERIOD_END.isoformat(),
+            )
+
+    def test_fetch_all_build_query_without_query_function(self):
+        query = (
+            'avg(delta(http_requests_total'
+            '{project_id="f266f30b11f246b589fd266f85eeec39"}[3600s]'
+            ')) by (foo, bar, project_id, code, instance)'
+        )
+
+        with mock.patch.object(
+            prometheus.PrometheusClient, 'get_instant',
+        ) as mock_get:
+            self.collector_without_query_function.fetch_all(
+                'http_requests_total',
+                samples.FIRST_PERIOD_BEGIN,
+                samples.FIRST_PERIOD_END,
+                self._tenant_id,
+            )
+            mock_get.assert_called_once_with(
+                query,
+                samples.FIRST_PERIOD_END.isoformat(),
+            )
+
+    def test_fetch_all_build_query_all(self):
+        query = (
+            'avg(abs(delta(http_requests_total'
+            '{project_id="f266f30b11f246b589fd266f85eeec39"}[3600s]'
+            '))) by (foo, bar, project_id, code, instance)'
+        )
+
+        with mock.patch.object(
+            prometheus.PrometheusClient, 'get_instant',
+        ) as mock_get:
+            self.collector_all.fetch_all(
+                'http_requests_total',
+                samples.FIRST_PERIOD_BEGIN,
+                samples.FIRST_PERIOD_END,
+                self._tenant_id,
+            )
+            mock_get.assert_called_once_with(
+                query,
+                samples.FIRST_PERIOD_END.isoformat(),
             )
 
     def test_format_data_instant_query(self):
@@ -96,7 +231,7 @@ class PrometheusCollectorTest(tests.TestCase):
             'end': samples.FIRST_PERIOD_END,
             'data': samples.PROMETHEUS_RESP_INSTANT_QUERY['data']['result'][0],
         }
-        actual = self.collector._format_data(**params)
+        actual = self.collector_mandatory._format_data(**params)
         self.assertEqual(expected, actual)
 
     def test_format_data_instant_query_2(self):
@@ -117,38 +252,21 @@ class PrometheusCollectorTest(tests.TestCase):
             'end': samples.FIRST_PERIOD_END,
             'data': samples.PROMETHEUS_RESP_INSTANT_QUERY['data']['result'][1],
         }
-        actual = self.collector._format_data(**params)
+        actual = self.collector_mandatory._format_data(**params)
         self.assertEqual(expected, actual)
 
     def test_format_retrieve(self):
-        expected = {
-            'http_requests_total': [
-                {
-                    'desc': {
-                        'bar': '', 'foo': '', 'project_id': '',
-                        'code': '200', 'instance': 'localhost:9090',
-                    },
-                    'groupby': {'bar': '', 'foo': '', 'project_id': ''},
-                    'metadata': {'code': '200', 'instance': 'localhost:9090'},
-                    'vol': {
-                        'qty': Decimal('7'),
-                        'unit': 'instance'
-                    }
-                },
-                {
-                    'desc': {
-                        'bar': '', 'foo': '', 'project_id': '',
-                        'code': '200', 'instance': 'localhost:9090',
-                    },
-                    'groupby': {'bar': '', 'foo': '', 'project_id': ''},
-                    'metadata': {'code': '200', 'instance': 'localhost:9090'},
-                    'vol': {
-                        'qty': Decimal('42'),
-                        'unit': 'instance'
-                    }
-                }
-            ]
-        }
+        expected_name = 'http_requests_total'
+        expected_data = [
+            dataframe.DataPoint(
+                'instance', '7', '0',
+                {'bar': '', 'foo': '', 'project_id': ''},
+                {'code': '200', 'instance': 'localhost:9090'}),
+            dataframe.DataPoint(
+                'instance', '42', '0',
+                {'bar': '', 'foo': '', 'project_id': ''},
+                {'code': '200', 'instance': 'localhost:9090'}),
+        ]
 
         no_response = mock.patch(
             'cloudkitty.common.prometheus_client.PrometheusClient.get_instant',
@@ -156,7 +274,7 @@ class PrometheusCollectorTest(tests.TestCase):
         )
 
         with no_response:
-            actual = self.collector.retrieve(
+            actual_name, actual_data = self.collector_mandatory.retrieve(
                 metric_name='http_requests_total',
                 start=samples.FIRST_PERIOD_BEGIN,
                 end=samples.FIRST_PERIOD_END,
@@ -164,7 +282,8 @@ class PrometheusCollectorTest(tests.TestCase):
                 q_filter=None,
             )
 
-        self.assertEqual(expected, actual)
+        self.assertEqual(expected_name, actual_name)
+        self.assertEqual(expected_data, actual_data)
 
     def test_format_retrieve_raise_NoDataCollected(self):
         no_response = mock.patch(
@@ -175,7 +294,7 @@ class PrometheusCollectorTest(tests.TestCase):
         with no_response:
             self.assertRaises(
                 collector.NoDataCollected,
-                self.collector.retrieve,
+                self.collector_mandatory.retrieve,
                 metric_name='http_requests_total',
                 start=samples.FIRST_PERIOD_BEGIN,
                 end=samples.FIRST_PERIOD_END,
@@ -192,7 +311,7 @@ class PrometheusCollectorTest(tests.TestCase):
         with invalid_response:
             self.assertRaises(
                 exceptions.CollectError,
-                self.collector.retrieve,
+                self.collector_mandatory.retrieve,
                 metric_name='http_requests_total',
                 start=samples.FIRST_PERIOD_BEGIN,
                 end=samples.FIRST_PERIOD_END,
